// ****************************************************************************
// * FILE NAME   :  TutorialTop.v
// * DATA        :  March 14, 2006
// * VERSION     :  1.1.0
// * COMPLIANCE  :  Verilog 2001 
// *
// * DESCRIPTION :  Top-level instantiation of subsystem generated by 
// *                CoreConsole.
// *
// * MODIFICATION HISTORY:
// *  14 NOV 2005  1.0     Initial Release
// *  17 JAN 2006  1.0.1   Modified to Support CoreConsole 1.0.1 & Libero 7.0
// *  14 MAR 2006  1.1.0   Modified to Support REV2 of CoreMP7 Development Kit
// ****************************************************************************

`timescale 1ns/100ps

// ****************************************************************************
// * The user must create this top-level file which instantiates the 
// * CoreConsole generated subsystem.
// *
// * Any bi-directional signals must be handled at this level, as CoreConsole
// * currently is not capable of dealing with tri-states.
// ****************************************************************************

module TutorialTop (
        SYSCLK,
        NSYSRESET,
        
        RemapDefault,
        HIGH,
        LOW,
        
        FLASH_BYTEN,
        FLASH_CSN,
        FLASH_OEN,
        FLASH_RPN,
        FLASH_WEN,
        
        SRAM_ADSC,
        SRAM_ADSP,
        SRAM_ADV,
        SRAM_BYTE_WEN,
        SRAM_BYTEN,
        SRAM_CLK,
        SRAM_CSN,
        SRAM_GLOBAL_WEN,
        SRAM_OEN,
        SRAM_PWRDWN,

        MEM_ADDR,
        MEM_DATA,
        
        SW,
        LED,
        
        RX0,
        TX0,

        ICE_nTRST,
        ICE_TCK,
        ICE_TDI,
        ICE_TMS,
        ICE_VTref,
        ICE_TDO,
        ICE_RTCK,
        ICE_nSRST,
        ICE_DBGACK,
        ICE_DBGRQ        
);



  // System Inputs
  input         SYSCLK;      // External clock input
  input         NSYSRESET;   // External reset in

  // Default Value for ReMap Signal
  input         RemapDefault;
  output        HIGH;
  output        LOW;

  // Flash Memory Interface
  output        FLASH_BYTEN; // Flash byte mode enable
  output        FLASH_CSN;   // Flash chip select
  output        FLASH_OEN;   // Flash chip output enable
  output        FLASH_RPN;   // Flash reset/block temporary unprotect
  output        FLASH_WEN;   // Flash chip byte write enable

  // Synchronous SRAM Interface
  output        SRAM_ADSC;          // SRAM chip address strobe (cache)
  output        SRAM_ADSP;          // SRAM chip address strobe (processor)
  output        SRAM_ADV;           // SRAM chip burst address enable
  output        SRAM_CLK;           // SRAM chip clock
  output        SRAM_CSN;           // SRAM chip select
  output [ 3:0] SRAM_BYTEN;         // SRAM byte 0..3 enable
  output        SRAM_BYTE_WEN;      // SRAM chip byte write enable
  output        SRAM_GLOBAL_WEN;    // SRAM chip global write enable
  output        SRAM_PWRDWN;        // SRAM chip power-down enable
  output        SRAM_OEN;           // SRAM chip output enable

  // Common Memory Interface Signals
  output [18:0] MEM_ADDR;    // Flash/SRAM Address Bus
  inout  [31:0] MEM_DATA;    // Flash/SRAM Data Bus

  // GPIO Interface - Switches & LEDs
  input  [ 7:0] SW;          // Switch Inputs
  output [ 7:0] LED;         // LED Outputs

  // RS232 Transceiver
  input         RX0;
  output        TX0;

  // CoreMP7 JTAG Interface
  input         ICE_nTRST;   // RealView ICE JTAG reset
  input         ICE_TCK;     // RealView ICE JTAG clock enable
  input         ICE_TDI;     // RealView ICE JTAG data in
  input         ICE_TMS;     // RealView ICE JTAG mode select
  output        ICE_VTref;   // RealView ICE target reference voltage
  output        ICE_TDO;     // RealView ICE JTAG data out
  output        ICE_RTCK;    // RealView ICE RTCK (used for adaptive clocking)
  inout         ICE_nSRST;   // RealView ICE System Reset
  output        ICE_DBGACK;  // RealView ICE Debug Acknowledge (Not Used)
  output        ICE_DBGRQ;   // RealView ICE Debug Request (Not Used)


// *****************************************************************************
// * SIGNAL DECLARATIONS
// *****************************************************************************
  wire   [31:0] MEM_DATA_IN;         // Flash/SRAM Data Bus Input
  wire   [31:0] MEM_DATA_OUT;        // Flash/SRAM Data Bus Output
  wire          MEM_DATA_OEN;        // Flash/SRAM Data Bus Output Enable
  wire   [27:0] MEM_ADDR_INT;        // Flash/SRAM Address Bus (Internal) 

  wire          ICE_nSRST_IN;        // RealView ICE System Reset Input
  wire          ICE_TDOUT;           // RealView ICE Test Data Out
  wire          ICE_nTDOEN;          // RealView ICE Test Data Out Enable

  wire  [31:0]  keyPadIn;            // CoreMP7 Evaluation Board Switches
  wire  [31:0]  ledOut;              // CoreMP7 Evaluation Board LEDs

  wire          HCLK;                // CoreMP7 Synchronized Clock 
  wire          SYSCLK_DIV4;         // CoreMP7 System Clock (Divided SYSCLK)


// *****************************************************************************
// * REGISTER DECLARATIONS
// *****************************************************************************
   reg  [ 2:0]  iCOUNT;


// *****************************************************************************
// * SIGNAL ASSIGNMENTS
// *****************************************************************************
   assign SYSCLK_DIV4 = iCOUNT[1];

   assign HIGH = 1'b1;
   assign LOW  = 1'b0;

   // Flash Memory Configuration Signals
   assign FLASH_BYTEN = 1'b1;        // Flash Byte Mode Enable (16-Bit Mode)
   assign FLASH_RPN   = NSYSRESET;   // Flash Reset Block Temporary Unprotect

   // Synchronous SRAM Configuration Signals
   assign SRAM_CLK          = ~(HCLK);  // SRAM Clock Input Signal
   assign SRAM_CSN          = 1'b0;
   assign SRAM_PWRDWN       = 1'bz;  // SRAM Sleep Mode Control (on-chip Pull-down, Active High)
   assign SRAM_ADV          = 1'b1;  // SRAM Burst Address Counter Enable (Active Low)
   assign SRAM_ADSP         = 1'b1;  // SRAM Address Strobe Processor (Active Low)
   assign SRAM_ADSC         = 1'b0;  // SRAM Address Strobe Controller (Active Low)
   assign SRAM_GLOBAL_WEN   = 1'b1;  // SRAM Global Write Enable (Active Low)
                                     //  use Write Enable to control writes                                      

   // When RV_nTDOEN is low, data is being driven on RV_TDOUT
   assign ICE_TDO = (ICE_nTDOEN == 1'b0) ? ICE_TDOUT : 1'bz;

   // Generate the bi-directional RV_nSRST signal buffer.
   // NOTE:  This has to act like an open-collector output, so it only ever drives
   //        out a low signal. Otherwise, it goes high-impedance, and is pulled 
   //        high by an external resistor.
   assign ICE_nSRST = (NSYSRESET == 1'b0) ? 1'b0 : 1'bz;
   assign ICE_nSRST_IN = ICE_nSRST;

   // Tie RealView target reference voltage signal high. This signal indicates 
   //   that the target has power and must be at least 0.628 V.
   assign ICE_VTref = 1'b1;

   // Tie unused RealView signals low
   assign ICE_DBGACK = 1'b0;
   assign ICE_DBGRQ  = 1'b0;

   // Switch Inputs
   assign keyPadIn[31:8] = 24'b0;
   assign keyPadIn[ 7:0] = SW;

  // LED Outputs
   assign LED = ledOut[7:0];

  // Bi-directional Memory Data Bus
  generate
   genvar i;
   for (i = 0; i < 32; i = i + 1)
    begin : MEM_DATA_GENERATE
     assign MEM_DATA[i] = (MEM_DATA_OEN == 1'b0) ? MEM_DATA_OUT[i] : 1'bz;
    end
  endgenerate

  assign MEM_DATA_IN = MEM_DATA;
  assign MEM_ADDR = MEM_ADDR_INT[18:0];


// ****************************************************************************
// * INSTANTIATION OF SUBSYSTEM
// ****************************************************************************
   TutorialMP7 TutorialTop (
     .HCLK               (HCLK),
     .Mem_FlashCSN       (FLASH_CSN),
     .Mem_FlashOEnN      (FLASH_OEN),
     .Mem_FlashWEnN      (FLASH_WEN),
     .Mem_MemAddr        (MEM_ADDR_INT),
     .Mem_MemDataIn      (MEM_DATA_IN),
     .Mem_MemDataOEnN    (MEM_DATA_OEN),
     .Mem_MemDataOut     (MEM_DATA_OUT),
     .Mem_MemReadN       (),
     .Mem_MemWriteN      (),

     .Mem_SramByte0N     (SRAM_BYTEN[0]),
     .Mem_SramByte1N     (SRAM_BYTEN[1]),
     .Mem_SramByte2N     (SRAM_BYTEN[2]),
     .Mem_SramByte3N     (SRAM_BYTEN[3]),
     .Mem_SramWEnN       (SRAM_BYTE_WEN),

     .Mem_SramCSN        (),
     .Mem_SramOEnN       (SRAM_OEN),
     .NSYSRESET          (NSYSRESET),
     .RV_RV_RTCK         (ICE_RTCK),
     .RV_RV_TCK          (ICE_TCK),
     .RV_RV_TDI          (ICE_TDI),
     .RV_RV_TDOUT        (ICE_TDOUT),
     .RV_RV_TMS          (ICE_TMS),
     .RV_RV_nSRST_IN     (ICE_nSRST_IN),
     .RV_RV_nTDOEN       (ICE_nTDOEN),
     .RV_RV_nTRST        (ICE_nTRST),
     .ReMapDef           (RemapDefault),
     .SYSCLK             (SYSCLK_DIV4),
     .UART_RX            (RX0),
     .UART_TX            (TX0),
     .keyPadIn           (keyPadIn),
     .ledOut             (ledOut)
   );

   always @ (posedge SYSCLK)
    begin
     if ( NSYSRESET == 1'b0 )
      iCOUNT = 2'b00;
     else iCOUNT = iCOUNT + 1'b1;
    end

endmodule

// *********************************** END ************************************
